@@ -24,6 +24,9 @@
 /* enable keyboard LED blinking? */
 #undef USE_LED
 
+/* build as a GNOME Applet */
+#undef GNOME_APPLET
+
 /* Does your system have the snprintf() call? */
 #undef HAVE_SNPRINTF
 
