@@ -6,7 +6,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <strings.h>
+#include <string.h>
 #include <errno.h>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -118,7 +118,6 @@ void gbuffy_save_conf ()
   proplist_t newsrc = PLMakeString ("newsrc");
   proplist_t tmp;
   BOX_INFO *box;
-  int row = 0;
   char buf[STRING];
 
   top = PLMakeDictionaryFromEntries (NULL, NULL);
@@ -201,9 +200,11 @@ void gbuffy_save_conf ()
   top = pl_set_dict_entry (top, "vertical", (Vertical ? "true" : "false"));
   snprintf (buf, sizeof (buf), "%d", PollTime);
   top = pl_set_dict_entry (top, "polltime", buf);
+  snprintf (buf, sizeof (buf), "%d", Rows);
+  top = pl_set_dict_entry (top, "rows", buf);
   top = pl_set_dict_entry (top, "maildir", NONULL(Maildir));
 
-  file = PLMakeString (DefaultsFile);
+  file = PLMakeString (get_defaults_file());
 
   top = PLSetFilename (top, file);
   PLSave (top, TRUE);
@@ -311,7 +312,6 @@ BOX_INFO *configure_load_boxes (proplist_t boxes)
 BOX_INFO *gbuffy_configure_load ()
 {
   proplist_t top;
-  proplist_t file;
   proplist_t boxes;
   proplist_t tmp;
   BOX_INFO *box;
@@ -347,6 +347,12 @@ BOX_INFO *gbuffy_configure_load ()
       s = PLGetString (tmp);
       sscanf (s, "%d", &PollTime);
     }
+    tmp = pl_get_dict_entry (top, "rows");
+    if (tmp)
+    {
+      s = PLGetString (tmp);
+      sscanf (s, "%d", &Rows);
+    }
     Maildir = configure_load_path_with_default (top, "maildir", "~/Mail");
 
     box = configure_load_boxes (boxes);
@@ -377,7 +383,7 @@ static char *get_defaults_file ()
   {
     if (errno != ENOENT)
     {
-      fprintf (stderr, "-E- Error accessing path %s\n");
+      fprintf (stderr, "-E- Error accessing path %s\n", path);
       perror ("stat");
     }
     strfcpy (path, "~/.gbuffyrc", sizeof (path));    
@@ -391,7 +397,7 @@ static char *get_defaults_file ()
     {
       if (errno != ENOENT)
       {
-	fprintf (stderr, "-E- Error accessing path %s\n");
+	fprintf (stderr, "-E- Error accessing path %s\n", path);
 	perror ("stat");
       }
       else
