@@ -6,6 +6,8 @@
 #include <pwd.h>
 #include <sys/time.h>
 #include <dirent.h>
+#include <errno.h>
+#include <ctype.h>
 
 #include "gbuffy.h"
 #include "rfc2047.h"
@@ -87,7 +89,6 @@ int mbox_folder_count (BOX_INFO *mbox, int force, GList *headers)
   char buffer[STRING];
   char garbage[STRING];
   char path[_POSIX_PATH_MAX];
-  int in_header = TRUE;
   struct stat s;
   struct timeval t[2];
   
@@ -102,6 +103,12 @@ int mbox_folder_count (BOX_INFO *mbox, int force, GList *headers)
   {
     mbox->file_size = 0;
     mbox->file_mtime = 0;
+    if (mbox->num_messages)
+    {
+      mbox->num_messages = 0;
+      mbox->new_messages = 0;
+      return 1;
+    }
     return 0;
   }
   
@@ -157,17 +164,9 @@ int dir_folder_count (BOX_INFO *mbox, int force, GList *headers)
 {
   DIR *dp = 0;
   FILE *fp = 0;
-  char buffer[STRING];
-  char garbage[STRING];
-  char from[STRING] = "";
-  char subject[STRING] = "";
   char path[_POSIX_PATH_MAX];
   char file[_POSIX_PATH_MAX];
-  int found = FALSE;
-  int status = FALSE;
-  int is_new = FALSE;
   int mailfile = TRUE;
-  int count = 0;
   struct dirent *de;
   struct stat s;
   struct timeval t[2];
@@ -189,6 +188,12 @@ int dir_folder_count (BOX_INFO *mbox, int force, GList *headers)
   {
     mbox->file_size = 0;
     mbox->file_mtime = 0;
+    if (mbox->num_messages)
+    {
+      mbox->num_messages = 0;
+      mbox->new_messages = 0;
+      return 1;
+    }
     return 0;
   }
   
@@ -228,11 +233,7 @@ int dir_folder_count (BOX_INFO *mbox, int force, GList *headers)
       if (headers != NULL || mbox->type == GB_MH) 
       {
 	/* Ok, we need to get the From: and Subject: lines */
-	if (mbox->type == GB_MAILDIR) {
-	  snprintf (file, sizeof (file), "%s/new/%s", path, de->d_name);
-	} else {
-	  snprintf (file, sizeof (file), "%s/%s", path, de->d_name);
-	}
+	snprintf (file, sizeof (file), "%s/%s", path, de->d_name);
 	fp = fopen (file, "r");
 	if (fp != NULL) 
 	{
@@ -265,4 +266,5 @@ int dir_folder_count (BOX_INFO *mbox, int force, GList *headers)
 
 int external_folder_count (BOX_INFO *mbox, int force, GList *headers)
 {
+  return 0;
 }
