@@ -3,7 +3,8 @@
  */
 
 #include <stdio.h>
-#include <strings.h>
+#include <stdlib.h>
+#include <string.h>
 #include <errno.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
@@ -24,6 +25,7 @@ static GtkWidget *TitleEntry;
 static GtkWidget *PathEntry;
 static GtkWidget *CommandEntry;
 static GtkWidget *PollTimeEntry;
+static GtkWidget *RowsEntry;
 static GtkWidget *MaildirEntry;
 static GtkWidget *MailboxCList;
 static GtkWidget *TypeOptionMenu;
@@ -130,7 +132,7 @@ static void configure_select_mailbox (GtkWidget *clist, gint row, gint column,
   gtk_option_menu_set_history (GTK_OPTION_MENU (TypeOptionMenu), box->type);
 }
 
-static configure_set_box (BOX_INFO *box)
+static void configure_set_box (BOX_INFO *box)
 {
   if (box->title)
     free (box->title);
@@ -316,6 +318,13 @@ static void configure_close (GtkWidget *button, gpointer data)
   safe_free ((void *)&Maildir);
   Maildir = safe_strdup (gtk_entry_get_text (GTK_ENTRY (MaildirEntry)));
   PollTime = atoi (gtk_entry_get_text (GTK_ENTRY (PollTimeEntry)));
+  row = atoi (gtk_entry_get_text (GTK_ENTRY (RowsEntry)));
+  if (row != Rows)
+  {
+    Rows = row;
+    Height = -1;
+    Width = -1;
+  }
 
   gtk_widget_hide (ConfigureWindow);
   gbuffy_display ();
@@ -398,7 +407,6 @@ static void configure_save (GtkWidget *button, gpointer data)
 {
   BOX_INFO *box;
   int row = 0;
-  char buf[STRING];
 
   box = MailboxInfo;
   while (box != NULL)
@@ -414,6 +422,13 @@ static void configure_save (GtkWidget *button, gpointer data)
   safe_free ((void *)&Maildir);
   Maildir = safe_strdup (gtk_entry_get_text (GTK_ENTRY (MaildirEntry)));
   PollTime = atoi (gtk_entry_get_text (GTK_ENTRY (PollTimeEntry)));
+  row = atoi (gtk_entry_get_text (GTK_ENTRY (RowsEntry)));
+  if (row != Rows)
+  {
+    Rows = row;
+    Height = -1;
+    Width = -1;
+  }
 
   gbuffy_save_conf ();
 }
@@ -456,7 +471,6 @@ void gbuffy_configure_dialog ()
   GtkWidget *vbox;
   GtkWidget *table;
   GtkWidget *label;
-  GtkWidget *entry;
   GtkWidget *button;
   GtkWidget *frame;
   GtkWidget *scroll;
@@ -647,6 +661,18 @@ void gbuffy_configure_dialog ()
     gtk_widget_show (PollTimeEntry);
     gtk_widget_show (table);
 
+    label = gtk_label_new ("Rows:");
+    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 2, 3, 
+	GTK_FILL, GTK_EXPAND, 0, 5);
+    gtk_widget_show (label);
+    RowsEntry = gtk_entry_new ();
+    snprintf (buf, sizeof (buf), "%d", Rows);
+    gtk_entry_set_text (GTK_ENTRY (RowsEntry), buf);
+    gtk_table_attach (GTK_TABLE (table), RowsEntry, 3, 4, 2, 3, 
+	GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 5);
+    gtk_widget_show (RowsEntry);
+    gtk_widget_show (table);
+
     button = gtk_button_new_with_label ("Insert");
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (ConfigureWindow)->action_area), 
 	button, TRUE, TRUE, 0);
