@@ -4,12 +4,21 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <gtk/gtk.h>
 #include <pwd.h>
 #include <string.h>
+#include <compface.h>
 
 #include "gbuffy.h"
 
+#define NEGATIVE -1
+#define POSITIVE 1
+
+#ifdef GNOME_APPLET
+#include <applet-widget.h>
+#endif
+
 char *Homedir;
 char *Username;
 char *Maildir = NULL;
@@ -19,6 +28,20 @@ int Vertical = FALSE;
 int PollTime = 10;
 int PollId = 0;
 int DontExit = FALSE;
+int Rows = 1;
+
+struct 
+{
+  gint value;
+  gint sign;
+}  HorizPos, VertPos;
+
+gint Width = -1;
+gint Height = -1;
+gint ScrH;
+gint ScrW;
+GtkWidget *MainWindow = NULL;
+
 #ifdef DEBUG
 FILE *debugfile = NULL;
 int debuglevel = 0;
@@ -27,8 +50,6 @@ int debuglevel = 0;
 /* This might be an option someday */
 int NoHeaderStay = TRUE;
 
-static GtkStyle *SelectedStyle = 0;
-
 BOX_INFO *MailboxInfo;
 /* This currently needs to be in the same order as enum BOXTYPE */
 BOX_CLASS MailboxClass[] =
@@ -65,7 +86,6 @@ GdkPixmap * pixmap_from_ikon (GdkWindow *w, char *buf)
 {
   char data[300];
   unsigned int a = 0, b = 0, c = 0;
-  int num = 0;
   int x = 0;
   char *line;
   char *p;
@@ -155,7 +175,7 @@ gdk_draw_bitmap (GdkDrawable *drawable,
 
 gint configure_event (GtkWidget *da, GdkEventConfigure *event, BOX_INFO *box)
 {
-  GList *headers, *tmp;
+  GList *headers;
   int y = box->height;
   int x;
   int face_display = FALSE;
@@ -233,8 +253,7 @@ gint configure_event (GtkWidget *da, GdkEventConfigure *event, BOX_INFO *box)
 
 gint populate_clist (GtkWidget *clist, BOX_INFO *box)
 {
-  GList *headers, *tmp;
-  int y = box->height;
+  GList *headers;
   int x;
   char buf[STRING];
   int row = 0;
@@ -290,7 +309,6 @@ gint populate_clist (GtkWidget *clist, BOX_INFO *box)
 void gbuffy_display_box (BOX_INFO *box, int xpos, int ypos)
 {
   GtkWidget *da;
-  GtkWidget *clist;
   GList *headers;
   char buf[STRING];
   int height = 16, width = 0;
@@ -394,27 +412,38 @@ void gbuffy_display_box (BOX_INFO *box, int xpos, int ypos)
 gint gbuffy_button_callback (GtkWidget *widget, GdkEventButton *event, 
     BOX_INFO *box)
 {
-  switch (event->button)
-  {
-    case 1:
-      box->leave = FALSE;
-      gbuffy_display_box (box, event->x_root, event->y_root);
-      break;
-    case 2:
-      if (box->command != NULL)
-	gbuffy_system (box->command, GB_DETACH_PROCESS);
-      break;
-    case 3:
-      gtk_timeout_remove (PollId);
-      gbuffy_configure_dialog ();
-      break;
-    default:
-      break;
+  switch (event->button) {
+  case 1:
+    box->leave = FALSE;
+    gbuffy_display_box (box, event->x_root, event->y_root);
+    break;
+  case 2:
+    if (box->command != NULL)
+      gbuffy_system (box->command, GB_DETACH_PROCESS);
+    break;
+  case 3:
+#ifdef GNOME_APPLET
+    return FALSE;
+#else
+    gtk_timeout_remove (PollId);
+    gbuffy_configure_dialog ();
+    break;
+#endif
+  default:
+    break;
   }
-/*  gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");  */
   return TRUE;
 }
 
+#ifdef GNOME_APPLET
+static void
+gbuffy_properties_callback (AppletWidget* applet, gpointer data)
+{
+  gtk_timeout_remove(PollId);
+  gbuffy_configure_dialog();
+}
+#endif
+
 gint gbuffy_leave_box (GtkWidget *widget, GdkEventButton *event, 
     BOX_INFO *box)
 {
@@ -459,6 +488,8 @@ gint gbuffy_poll_boxes (gpointer data)
 {
   BOX_INFO *mbox;
   int change;
+  gint cur_width;
+  gint cur_height;
 
   mbox = MailboxInfo;
   while (mbox != NULL)
@@ -478,43 +509,150 @@ gint gbuffy_poll_boxes (gpointer data)
     }
     mbox = mbox->next;
   }
+
+  /* This is the wrong way to do this, but its quick and dirty.  I
+   * should figure out what event I should capture to receive and update
+   * this information, but instead we just update it here. */
+
+  gdk_window_get_size(MainWindow->window, &cur_width, &cur_height);
+  if (cur_width != Width)
+    Width = cur_width;
+  if (cur_height != Height)
+    Height = cur_height;
+  
+  HorizPos.sign = POSITIVE;
+  VertPos.sign = POSITIVE;
+
+  gdk_window_get_position (MainWindow->window, &HorizPos.value, &VertPos.value);
+
   return TRUE;
 }
 
-int gbuffy_display ()
+#ifdef GNOME_APPLET
+/*this is when the panel orientation changes*/
+static void applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
+{
+  int oldVertical = Vertical;
+
+  switch(o) {
+    case ORIENT_UP:
+    case ORIENT_DOWN:
+      Vertical = FALSE;
+      break;
+    case ORIENT_LEFT:
+    case ORIENT_RIGHT:
+      Vertical = TRUE;
+      break;
+  }
+  if(Vertical != oldVertical)
+    gbuffy_display();
+}
+
+static void about_cb (AppletWidget* applet, gpointer data) 
+{
+  const gchar *authors[] = {
+    "Brandon Long",
+    "Tim Danner",
+    NULL
+  };
+  GtkWidget* a;
+
+  a = gnome_about_new ("GBuffy", VERSION,
+      "Copyright 1999 Brandon Long",
+      (const gchar **) authors,
+      _("GBuffy monitors mail in many formats, including "
+	"IMAP, plus newsgroups."),
+      NULL);
+  gtk_widget_show (a);
+}
+#endif
+
+void gbuffy_display ()
 {
-  static GtkWidget *window = NULL;
   GtkWidget *box;
-  GtkWidget *button;
   BOX_INFO *mbox;
   char buf[STRING];
+  int box_count = 0, columns, rows;
+#ifdef GNOME_APPLET
+  GList* children;
+#endif
 
-  if (window != NULL)
+  gint cur_width;
+  gint cur_height;
+  gint new_horizontal;
+  gint new_vertical;
+
+#ifdef GNOME_APPLET
+  if (MainWindow != NULL) {
+    children = gtk_container_children (GTK_CONTAINER (MainWindow));
+    gtk_widget_destroy (GTK_WIDGET (children->data));
+  } else {
+    MainWindow = applet_widget_new ("gbuffy");
+    applet_widget_register_stock_callback(APPLET_WIDGET(MainWindow),
+		  "properties",
+		  GNOME_STOCK_MENU_PROP,
+		  _("Properties..."),
+		  gbuffy_properties_callback,
+		  NULL);
+    applet_widget_register_stock_callback(APPLET_WIDGET(MainWindow),
+		  "about",
+		  GNOME_STOCK_MENU_ABOUT,
+		  _("About..."),
+		  about_cb,
+		  NULL);
+  }
+#else
+  if (MainWindow != NULL)
   {
     DontExit = TRUE;
-    gtk_widget_destroy (window);
+    gtk_widget_destroy (MainWindow);
+  }
+  MainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+#endif
+
+  gtk_widget_set_name (MainWindow, "main MainWindow");
+
+  gtk_signal_connect (GTK_OBJECT (MainWindow), "delete_event",
+      GTK_SIGNAL_FUNC (gbuffy_exit), NULL);
+  gtk_signal_connect (GTK_OBJECT (MainWindow), "destroy",
+      GTK_SIGNAL_FUNC (gbuffy_exit), NULL);
+
+  for (mbox = MailboxInfo; mbox; mbox = mbox->next)
+  {
+    box_count++;
   }
 
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_widget_set_name (window, "main window");
-/*  gtk_widget_set_uposition (window, 0, 0); */
-/*  gtk_container_border_width (GTK_CONTAINER (window), 0); */
-
-  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
-      GTK_SIGNAL_FUNC (gbuffy_exit), NULL);
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
-      GTK_SIGNAL_FUNC (gbuffy_exit), NULL);
+  rows = Rows;
 
+  columns = box_count / rows;
+  while (rows * columns < box_count) 
+  {
+    columns++;
+  }
   if (Vertical)
-    box = gtk_vbox_new (TRUE, 0);
-  else
-    box = gtk_hbox_new (TRUE, 0);
+  {
+    int tmp;
+    tmp = rows;
+    rows = columns;
+    columns = tmp;
+  }
 
-  gtk_container_add (GTK_CONTAINER (window), box);
+  box = gtk_table_new (rows, columns, TRUE);
+
+#ifdef GNOME_APPLET
+  gtk_signal_connect(GTK_OBJECT(MainWindow), "change_orient",
+		     GTK_SIGNAL_FUNC(applet_change_orient),
+		     NULL);
+  applet_widget_add (APPLET_WIDGET (MainWindow), box);
+#else
+  gtk_container_add (GTK_CONTAINER (MainWindow), box);
+#endif
 
   mbox = MailboxInfo;
+  box_count = 0;
   while (mbox != NULL)
   {
+    int x, y;
     /* (*(MailboxClass[mbox->type].count))(mbox, 1, NULL); */
     snprintf (buf, sizeof (buf), "%s: %d", mbox->title, mbox->new_messages);
     mbox->button = gtk_button_new_with_label (buf);
@@ -522,33 +660,120 @@ int gbuffy_display ()
     if (mbox->new_messages)
       gtk_widget_set_state (mbox->button, GTK_STATE_SELECTED);
 
-    gtk_box_pack_start (GTK_BOX (box), mbox->button, TRUE, TRUE, TRUE);
+    //gtk_box_pack_start (GTK_BOX (box), mbox->button, TRUE, TRUE, TRUE);
+    x = box_count / rows;
+    y = box_count % rows;
+    gtk_table_attach (GTK_TABLE (box), mbox->button,
+		      x, x + 1, y, y + 1, 
+		      GTK_FILL | GTK_SHRINK,
+		      GTK_FILL | GTK_SHRINK,
+		      0, 0);
     gtk_widget_show (mbox->button);
-    gtk_signal_connect (GTK_OBJECT (mbox->button), "button_press_event",
+    gtk_signal_connect_after (GTK_OBJECT (mbox->button), "button_press_event",
 	GTK_SIGNAL_FUNC (gbuffy_button_callback), mbox);
     gtk_signal_connect (GTK_OBJECT (mbox->button), "button_release_event",
 	GTK_SIGNAL_FUNC (gbuffy_delete_box), mbox);
     gtk_signal_connect_after (GTK_OBJECT (mbox->button), "leave_notify_event",
 	GTK_SIGNAL_FUNC (gbuffy_leave_box), mbox);
     mbox = mbox->next;
+    box_count++;
   }
 
   gtk_widget_show (box);
 
-  gtk_widget_show (window);
+  gtk_widget_show (MainWindow);
+  gdk_window_get_size(MainWindow->window, &cur_width, &cur_height);
+  Width = (Width == -1) ? cur_width : Width;
+  Height = (Height == -1) ? cur_height : Height;
+  gdk_window_resize(MainWindow->window, Width, Height);
+  
   gbuffy_poll_boxes (NULL);
 
   PollId = gtk_timeout_add (PollTime * 1000, gbuffy_poll_boxes, NULL);
+
+  if (!(HorizPos.sign != 0 && VertPos.sign != 0))
+    return;
+  new_horizontal = (HorizPos.sign == POSITIVE) ? HorizPos.value :
+    ScrW - Width - HorizPos.value;
+  new_vertical = (VertPos.sign == POSITIVE) ? VertPos.value :
+    ScrH - Height - VertPos.value;
+  gdk_window_move(MainWindow->window, new_horizontal, new_vertical);
+
 }
 
+void parse_geometry(int *argc, char ***argv)
+{
+  gint i, remove=0;
+  gchar *p;
+  
+  HorizPos.value=-1;
+  HorizPos.sign=0;
+  VertPos.value=-1;
+  VertPos.sign=0;
+
+  ScrH = gdk_screen_height();
+  ScrW = gdk_screen_width();
+
+    for (i=0;i<*argc;i++) {
+      if(strcmp("--geometry", (*argv)[i]) == 0 ||
+	 strncmp("--geometry", (*argv)[i], 10) == 0 ||
+	 strcmp("-geometry", (*argv)[i]) == 0 ||
+	 strncmp("-geometry", (*argv)[i], 9) == 0) {
+	if((i+1) == *argc) {/* '-(-)geometry' was the _last_ argument */
+	  (*argv)[i] = NULL;
+	  remove = -1;
+	  break;
+	}
+	p = (*argv)[i+1];
+	while(*p) {
+	  switch(*p) {
+	  case 'x': 
+	    Height = strtol(p + 1, &p, 10);
+	    break;
+	  case '-':
+	  case '+':
+	    if (HorizPos.value == -1) {
+	      HorizPos.sign = (*p == '-') ? NEGATIVE : POSITIVE;
+	      HorizPos.value = strtol(p+1, &p, 10);
+	    }
+	    else {
+	      VertPos.sign = (*p == '-') ? NEGATIVE : POSITIVE;
+	      VertPos.value = strtol(p+1, &p, 10);
+	    }
+	    break;
+	  default:
+	    Width = strtol(p, &p, 10);
+	    break;
+	  }
+	}	    /* while(p) */
+	(*argv)[i] = NULL;
+	(*argv)[i+1] = NULL;
+	break; /* We are done here */
+      }
+    }
+    if (remove == -1) 
+      argc--;
+    else 
+      for(i=1;i<*argc;i++) { /* Remove NULL:ed argv entries */
+	if ((*argv)[i] == NULL) {
+	  (*argv)[i] = (*argv)[i+2];
+	  remove++;
+	}
+      }
+    *argc-=remove;
+}        
+
 int main (int argc, char *argv[])
 {
   char path[_POSIX_PATH_MAX];
   char *p;
   struct passwd *pw;
 
+#ifdef GNOME_APPLET
+  applet_widget_init("gbuffy", VERSION, argc, argv, NULL, 0, NULL);
+#else
   gtk_init (&argc, &argv);
-
+#endif
 
   if ((p = getenv ("HOME")))
     Homedir = safe_strdup (p);
@@ -565,9 +790,10 @@ int main (int argc, char *argv[])
     if (!Homedir)
       Homedir = safe_strdup (pw->pw_dir);
   }
-
+  parse_geometry(&argc, &argv);
+  
 #ifdef DEBUG
-  if (argc == 3)
+  if (argc == 3) 
   {
     if (!strcmp (argv[1], "-d"))
     {
@@ -583,8 +809,12 @@ int main (int argc, char *argv[])
       g_print ("-I- Debugging at level %d\n", debuglevel);
     }
   }
+  g_print("Will set:\nWidth: %d\nHeight: %d\nXOFF: %d:%d\nYOFF: %d:%d\n",
+	  width, height, horizontal.sign, horizontal.value, vertical.sign, 
+	  vertical.value);
 #endif
 
+
   MailboxInfo = gbuffy_configure_load ();
   if (MailboxInfo == NULL)
   {
@@ -603,7 +833,11 @@ int main (int argc, char *argv[])
 
   gbuffy_display ();
 
+#ifdef GNOME_APPLET
+  applet_widget_gtk_main();
+#else
   gtk_main ();
+#endif
 
   return 0;
 }
