@@ -81,12 +81,16 @@ extern char *DefaultNewserver;
 extern char *Homedir;
 extern char *Maildir;
 extern int PollTime;
+extern int Rows;
 extern BOX_INFO *MailboxInfo;
 extern BOX_CLASS MailboxClass[];
 #ifdef DEBUG
 extern FILE *debugfile;
 extern int debuglevel;
 #endif
+extern gint Width;
+extern gint Height;
+
 
 #define FOREVER while (1)
 #define NONULL(x) x?x:""
@@ -108,6 +112,12 @@ int external_folder_count (BOX_INFO *mbox, int force, GList *headers);
 
 void gbuffy_configure_dialog ();
 BOX_INFO *gbuffy_configure_load ();
+void gbuffy_display ();
+void gbuffy_save_conf ();
+
+
+/* from.c */
+int is_from (const char *s, char *path, size_t pathlen);
 
 /* lib.c */
 void *safe_calloc (size_t nmemb, size_t size);
